package gov.va.claims.attachments.parser;

import gov.va.claims.attachments.model.ClaimsAttachment;
import gov.va.claims.attachments.parser.helpers.Context;
import gov.va.claims.attachments.parser.helpers.EDI;
import gov.va.claims.attachments.parser.helpers.FormatException;
import gov.va.claims.attachments.parser.helpers.Segment;
// import gov.va.claims.attachments.parser.InputParser;
// import gov.va.claims.attachments.parser.X275;
import gov.va.claims.attachments.parser.X275Simple;
// import gov.va.claims.attachments.parser.InputParser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Scanner;
import java.util.regex.Pattern;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;



/**
 * The class represents methods used to translate a X12 transaction represented
 * as a file or string into an X12 object.
 * 
 * 
 */
public class X275SimpleParser implements Parser {

	boolean DEBUG = true;
	
	static final Logger Log = LoggerFactory.getLogger(X275SimpleParser.class);
		
	static final int SIZE = 106;
	static final int POS_SEGMENT = 105;
	static final int POS_ELEMENT = 3;
	static final int POS_COMPOSITE_ELEMENT = 104;

	/**
	 * The method takes a X12 file and converts it into a X2 object. The X12
	 * class has methods to convert it into XML format as well as methods to
	 * modify the contents.
	 * 
	 * @param fileName
	 *            a X12 file
	 * @return the X12 object
	 * @throws FormatException
	 * @throws IOException
	 */
	@Override
	public EDI parse(File fileName) throws FormatException, IOException {
		
		boolean DEBUG = false;
		FileReader fr = null;
		
		final char[] buffer = new char[SIZE];
		
		try {
			fr = new FileReader(fileName);
		}catch(Exception e) {
			Log.error("X275SimpleParser.parse() - error reading input file: " + e.getMessage());
			e.printStackTrace();
		}
		
		
		int count = fr.read(buffer);
		
		fr.close();
		if (count != SIZE) {
			throw new FormatException();
		}
		
		Context context = new Context();
		context.setSegmentSeparator(buffer[POS_SEGMENT]);
		context.setElementSeparator(buffer[POS_ELEMENT]);
		context.setCompositeElementSeparator(buffer[POS_COMPOSITE_ELEMENT]);

		Character segmentSeparator = context.getSegmentSeparator();
		String quotedSegmentSeparator = Pattern.quote(segmentSeparator.toString());

		Scanner scanner = new Scanner(fileName);
		scanner.useDelimiter(quotedSegmentSeparator + "\r\n|" + quotedSegmentSeparator + "\n|" + quotedSegmentSeparator);

		X275Simple x275 = new X275Simple(context);
		while (scanner.hasNext()) {
			
			
// read one line at a time
			
			String line = scanner.next();
if(DEBUG) {			
System.out.println("X12SimpleParser.parse() - new scanned line is: " + line);			
}			
			
			x275.addSegment(line);
			
			if(DEBUG) {			
System.out.println("X12SimpleParser.parse() - appended segment is: " + x275.toString() ); 			
			}			
		}
		scanner.close();
		return x275;
		
	}	// end parse()

	/**
	 * The method takes a InputStream and converts it into a X2 object. The X12
	 * class has methods to convert it into XML format as well as methods to
	 * modify the contents.
	 * 
	 * @param source
	 *            InputStream
	 * @return the X12 object
	 * @throws FormatException
	 * @throws IOException
	 */
	@Override
	public EDI parse(InputStream source) throws FormatException, IOException {
		StringBuilder strBuffer = new StringBuilder();
		char[] cbuf = new char[1024];
		int length = -1;

		Reader reader = new BufferedReader(new InputStreamReader(source));

		while ((length = reader.read(cbuf)) != -1) {
			strBuffer.append(cbuf, 0, length);
		}

		String strSource = strBuffer.toString();

		return parse(strSource);
	}

	/**
	 * The method takes a X12 string and converts it into a X2 object. The X12
	 * class has methods to convert it into XML format as well as methods to
	 * modify the contents.
	 * 
	 * @param source
	 *            String
	 * @return the X12 object
	 * @throws FormatException
	 * @throws IOException
	 */
	@Override
	public EDI parse(String source) throws FormatException {
		
		if (source.length() < SIZE) {
			throw new FormatException();
		}
		
		
		Context context = new Context();
		
		context.setSegmentSeparator(source.charAt(POS_SEGMENT));
		context.setElementSeparator(source.charAt(POS_ELEMENT));
		context.setCompositeElementSeparator(source.charAt(POS_COMPOSITE_ELEMENT));

		Character segmentSeparator = context.getSegmentSeparator();
		String quotedSegmentSeparator = Pattern.quote(segmentSeparator.toString());

		Scanner scanner = new Scanner(source);
		scanner.useDelimiter(quotedSegmentSeparator + "\r\n|" + quotedSegmentSeparator + "\n|" + quotedSegmentSeparator);

		X275Simple x275 = new X275Simple(context);
		
	
		while (scanner.hasNext()) {
			String line = scanner.next();
			Segment s = x275.addSegment();
			String[] tokens = line.split("\\" + context.getElementSeparator());
			s.addElements(tokens);

			if(DEBUG) {			
// System.out.println("X12SimpleParser.parse() - Segment is now: " + s.toString());			
			}			
			
			
		}
		
		scanner.close();
		
		return (EDI) x275;
	}

} // end class
