
package gov.va.claims.attachments.parser.helpers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * This class represents an X12 segment.
 * 
 */
public class Segment implements Iterable<String> {
	
	private boolean DEBUG = false;
	private static final long serialVersionUID = 1L;
	private static final String EMPTY_STRING = "";
	
	private Context context;
	private List<String> elements = new ArrayList<String>();

	/**
	 * The constructor takes a <code>Context</code> object as input. The context
	 * object represents the delimiters in a X12 transaction.
	 * 
	 * @param c
	 *            the context object
	 */
	public Segment(Context c) {
		this.context = c;
	}

	/**
	 * Adds <code>String</code> element to the segment. The element is added at
	 * the end of the elements in the current segment.
	 * 
	 * @param e
	 *            the element to be added
	 * @return boolean
	 */
	public boolean addElement(String e) {
		
		
		if(DEBUG) {
			System.out.println("Segment.addElememnt() - element is: " + e);
		}		
		
		return elements.add(e);
	}

	/**
	 * Adds <code>String</code> with elements to the segment. The elements are
	 * added at the end of the elements in the current segment. e.g.
	 * <code>addElements("ISA*ISA01*ISA02");</code>
	 * 
	 * @param s
	 * @return boolean
	 */
	public boolean addElements(String s) {
		String[] elements = s.split("\\" + context.getElementSeparator());
		
		
		if(DEBUG) {
			System.out.println("Segment.addElememts() - element is: " + s);
		}		
		
		
		
		return this.addElements(elements);
	}

	/**
	 * Adds <code>String</code> elements to the segment. The elements are added
	 * at the end of the elements in the current segment. e.g.
	 * <code> addElements("ISA", "ISA01", "ISA02");</code>
	 * @param es
	 * @return boolean
	 */
	public boolean addElements(String... es) {
		
		
		for (String s : es) {
			
			if(DEBUG) {
				System.out.println("Segment.addElememts() - element is: " + s);
			}			
			
			
			
			if (!this.elements.add(s)) {
				return false;
			}
		}
		return true;
	}

	/**
	 * Adds strings as a composite element to the end of the segment.
	 * 
	 * @param ces
	 *            sub-elements of a composite element
	 * @return boolean
	 */
	public boolean addCompositeElement(String... ces) {
		StringBuilder dump = new StringBuilder();
		
		
		for (String s : ces) {
			
			if(DEBUG) {
				System.out.println("Segment.addCompositeElememt() - element is: " + ces);
			}
			
			
			
			
			dump.append(s);
			dump.append(context.getCompositeElementSeparator());
		}
		return this.elements.add(dump.substring(0, dump.length() - 1));
	}

	/**
	 * Inserts <code>String</code> element to the segment at the specified
	 * position
	 * 
	 * @param e
	 *            the element to be added
	 * @return boolean
	 */
	public boolean addElement(int index, String e) {
		
		
		if(DEBUG) {
			System.out.println("Segment.addElememt() - index and element are: " + index + ", " + e);
		}		
		
		
		return this.elements.add(e);
		
	} // 

	/**
	 * Inserts strings as a composite element to segment at specified position
	 * 
	 * @param ces
	 *            sub-elements of a composite element
	 */
	public void addCompositeElement(int index, String... ces) {
		StringBuilder dump = new StringBuilder();
		
		
		for (String s : ces) {
			dump.append(s);
			dump.append(context.getCompositeElementSeparator());
		}
		
		if(DEBUG) {
			System.out.println("Segment.addCompositeElememt() - new segment element is: " + elements.toString());
		}
		
		
		this.elements.add(index, dump.substring(0, dump.length() - 1));
	}

	/**
	 * Returns the context object
	 * 
	 * @return Context object
	 */
	public Context getContext() {
		return this.context;
	}

	/**
	 * Returns the <code>String<code> element at the specified position.
	 * 
	 * @param index
	 *            position
	 * @return the element at the specified position.
	 */
	public String getElement(int index) {
		return elements.get(index);
	}

	/**
	 * 
	 * @return List of elements
	 */
	public List<String> getElements() {
		return this.elements;
	}
	
	/**
	 * Returns and <code>Iterator</code> to the elements in the segment.
	 * 
	 * @return Iterator<String>
	 */
	@Override
	public Iterator<String> iterator() {
		return elements.iterator();
	}

	/**
	 * Removes the element at the specified position in this list.
	 * 
	 * @param index
	 * @return
	 */
	public String removeElement(int index) {
		return elements.remove(index);
	}

	/**
	 * Removes empty and null elements at the end of segment 
	 */
	private void removeTrailingEmptyElements() {
		
		
		for (int i = elements.size() - 1; i >= 0; i--) {
			if (elements.get(i) == null || elements.get(i).length() == 0) {
				elements.remove(i);
			} else {
				break;
			}
		}	
		
		
	}
	
	/**
	 * Sets the context of the segment
	 * 
	 * @param context
	 *            context object
	 */
	public void setContext(Context context) {
		this.context = context;
	}

	/**
	 * Replaces element at the specified position with the specified
	 * <code>String</code>
	 * 
	 * @param index
	 *            position of the element to be replaced
	 * @param s
	 *            new element with which to replace
	 */
	public void setElement(int index, String s) {
		elements.set(index, s);
	}

	/**
	 * Replaces composite element at the specified position in segment.
	 * 
	 * @param ces
	 *            sub-elements of a composite element
	 */
	public void setCompositeElement(int index, String... ces) {
		StringBuilder dump = new StringBuilder();
		for (String s : ces) {
			dump.append(s);
			dump.append(context.getCompositeElementSeparator());
		}
		elements.set(index, dump.substring(0, dump.length() - 1));
	}

	/**
	 * Returns number of elements in the segment.
	 * 
	 * @return size
	 */
	public int size() {
		return elements.size();
	}

	/**
	 * Returns the X12 representation of the segment.
	 */
	public String toString() {
		StringBuilder dump = new StringBuilder();
		for (String s : this.elements) {
			dump.append(s);
			dump.append(context.getElementSeparator());
		}
		if (dump.length() == 0) {
			return EMPTY_STRING;
		}
		return dump.substring(0, dump.length() - 1);
	}

	/**
	 * Returns the X12 representation of the segment.
	 * 
	 * @param bRemoveTrailingEmptyElements
	 * @return <code>String</code>
	 */
	public String toString(boolean bRemoveTrailingEmptyElements) {
		if (bRemoveTrailingEmptyElements)
			removeTrailingEmptyElements();
		return this.toString();
	}
	
	/**
	 * Returns the XML representation of the segment.
	 * 
	 * @return <code>String</code>
	 */
	public String toXML() {
		StringBuilder dump = new StringBuilder();
		dump.append("<" + this.elements.get(0) + ">");
		for (int i = 1; i < this.elements.size(); i++) {
			dump.append("<" + this.elements.get(0) + String.format("%1$02d", i) + "><![CDATA[");
			dump.append(this.elements.get(i));
			dump.append("]]></" + this.elements.get(0) + String.format("%1$02d", i) + ">");
		}
		dump.append("</" + this.elements.get(0) + ">");
		return dump.toString();
	}

	/**
	 * Returns the XML representation of the segment.
	 * 
	 * @param bRemoveTrailingEmptyElements
	 * @return <code>String</code>
	 */
	public String toXML(boolean bRemoveTrailingEmptyElements) {
		if (bRemoveTrailingEmptyElements)
			removeTrailingEmptyElements();
		return this.toXML();
	}

}
